#include "local.h"



/* global */
session_t _Session;
extern int gDebugFlags;


/*
==========
StartSession
==========
*/
int StartSession ( void ) {
	int ret;
	libusb_context *context;
	unsigned char buffer[32];

	ret = libusb_init( &context );
	_Session.context = context;
	if ( ret ) {
		EPrintf( TRUE, "Cannot initialise libUSB\n" );
		return FALSE;
	}

	libusb_set_debug( context, (gDebugFlags&DEBUG_EVENT)*3 ); /* max debug or none */

	ret = findDevice( VENDOR, PRODUCT );
	if ( !ret ) {
		EPrintf ( TRUE, "Could not initialise usb device.\n" );
		return FALSE;
	}

	ret = libusb_set_configuration(_Session.handle, 1);
	if ( ret ) {
		EPrintf( TRUE, "Cannot select configuration (1)\n" );
	}

	ret = libusb_claim_interface(_Session.handle, 0);
	if ( ret ) {
		EPrintf( TRUE, "Cannot claim interface (0)\n" );
	}

	ret = libusb_control_transfer( _Session.handle, 0xA0, 0x00, 0x0000, 0x0000, buffer, 0x0008, 1000);
	if ( ret < 0 ) {
		EPrintf( TRUE, "Request failed\n" );
	}
	else {
		/* We expect [0x00] [0x00] [0x00] [0x00] */
	}

	return TRUE;
}



/*
==========
findDevice
==========
*/
int findDevice( uint16_t vendor, uint16_t product ) {
	libusb_device **list;
	libusb_device *found=NULL;
	libusb_device *device;
	libusb_device_handle *handle;
	struct libusb_device_descriptor desc;
	ssize_t cnt;
	ssize_t i;
	int err, ret;

	cnt = libusb_get_device_list( _Session.context, &list );
	if ( cnt < 0 ) {
		EPrintf( TRUE, "No USB devices of any kind found\n" );
		return FALSE;
	}

	for ( i=0; i < cnt; i++ ) {
		device = list[i];

		ret = libusb_get_device_descriptor(device, &desc);
		if ( ret ) {
			EPrintf( TRUE, "Cannot read device descriptor\n" );
		}

		if ( (desc.idVendor == vendor) && (desc.idProduct == product) ) {
			found = device;
			DPrintf( DEBUG_USB, "Found Device vendor[0x%04x] product[0x%04x]\n", desc.idVendor, desc.idProduct );
			break;
		}
	}

	if ( found ) {
		err = libusb_open( found, &handle );
		if ( err ) {
			EPrintf( TRUE, "Cannot open USB device\n" );
			return FALSE;
		}
		_Session.handle = handle;
	}
	else {
		EPrintf(TRUE, "Cannot find required USB device\n");
	}

	libusb_free_device_list( list, TRUE );

	return TRUE;
}



/*
==========
ReceiveData
==========
*/
int ReceiveData( unsigned char *buffer, int len ) {
	int ret;
	int cnt;

	ret = libusb_interrupt_transfer 	( _Session.handle, 0x81, buffer, len, &cnt, 1000 );
	if ( ret ) {
		switch ( ret ) {
			case LIBUSB_ERROR_TIMEOUT:
				DPrintf( DEBUG_USB, "Interrupt transfer timeout\n" );
			break;
			case LIBUSB_ERROR_NO_DEVICE:
				EPrintf( TRUE, "Interrupt transfer failed. Device disconnected. [%i]\n", ret );
			break;
			default:
				EPrintf( TRUE, "Interrupt transfer somehow failed [%i]\n", ret );
			break;
		}
	}

	return cnt;
}



/*
==========
Dispose

Cleans up right before exit
==========
*/
void Dispose( void ) {
	if ( _Session.handle ) {
		libusb_close( _Session.handle );
	}

	if ( _Session.context ) {
		libusb_exit ( _Session.context );
	}
}

